/*
	utility program for the ProWesS loader

	MBUTTON : put a button in the button frame
	This program loads a descriptor file, which can contains several lines
	with names of programs which can be loaded. Each line will be for
	one program. It contains the name for the program to be displayed
	and the name for the loader file and possibly the path where the file
	should be searched.
	These lines are actually the same as the command line for the BUTTON
	command.

	When this button it indicated a menu with several programs which can
	be loaded by indicating them. When a program has been indicated, the
	mbutton program quits (but the program will be started.

	Each line of the input file can contain (same as button params) :
		-path XXX : searchpath for loader file
		-name XXX : name inside button
		-insert : ask for user to insert the disk first
		XXX : name of loader file which starts the program (obligatory)
	if no name is given, then the name of the loader file will be displayed
	The length of each line is limited to 160 characters.
	If a line starts with an exlamation mark (!), than the install program
	will be terminated after the loader is called to start that comand.

	MBUTTON has the filename of the file as parameter string. You can also pass
	the path which should be searched for the file. You can also pass the name
	for the window (preceded by "-name"). You can also pass "-wait", in that
	case the program will not terminate after starting the ProWesS loader.

	written by Joachim Van der Auwera for PROGS
	(c) 1996 by PROGS, Professional & Graphical Software
	started 30 March 1996
	31.08.1996 added "-terminate" option
*/

#include <io.h>
#include <str.h>
#include <thing.h>
#include <ProWesS.h>

#define catch(x) if (err=(x)) return err; else
Error select(PWObject object, char *item);

#define MAX_NAME 40
#define MAX_LINE 160

typedef struct {
	char name[MAX_NAME];		/* has to be first member !! */
	char *path, *file;
	char insert;
	char terminate;
} Line;

Error init(char *param)
{
	Error err;
	char **params;
	int i;
	PWObject obj,menu;
	char *file=NULL;
	char *name=NULL;
	char *path="$D;$P";
	char line[MAX_LINE];
	char wait=FALSE;
	Channel chid;

	/* parse the command line parameters */
	catch( STRBreak(param,&params) );
	for (i=0 ; params[i] ; i++)
	{
		if (STRSameCI("-path",params[i]))
		{
			if (params[i+1])
				path=params[++i];
			else
				return ERR_IPAR;
		}
		else
		if (STRSameCI("-name",params[i]))
		{
			if (params[i+1])
				name=params[++i];
			else
				return ERR_IPAR;
		}
		else
		if (STRSameCI("-wait",params[i]))
		{
			wait=TRUE;
		}
		else
			file=params[i];
	}
	if (!file) return ERR_IPAR;

	catch( IOOpenPath(file,OPEN_READ,path,&chid) );

	catch( PWCreate(NULL,&obj,PW_TYPE_OUTLINE,
					PW_OUTLINE_TITLE_TEXT, name,
					PW_OUTLINE_QUIT,
					PW_OUTLINE_QUIT_KEYPRESS, 27,
					PW_GLOBAL_AUXILIARY, &wait,
					NULL) );
	catch( PWCreate(obj,&menu,PW_TYPE_MENU,
					PW_MENU_NONE_SELECTED,
					PW_MENU_ACTION_SELECT, select,
					PW_OBJECT_AUTOSIZE,
					NULL) );

	while (ERR_EOF!=IOGetString(chid,line,MAX_LINE))
	{
		Line *this;

		catch( MEMAllocate(sizeof(Line),&this) );
		this->path="$D;$P";

		/* parse the command line parameters */
		catch( STRBreak(line,&params) );
		for (i=0 ; params[i] ; i++)
		{
			if (STRSameCI("-path",params[i]))
			{
				if (params[i+1])
					this->path=params[++i];
				else
					return ERR_IPAR;
			}
			else
			if (STRSameCI("-terminate",*params))
				this->terminate=TRUE;
			else
			if (STRSameCI("-name",params[i]))
			{
				if (params[i+1])
					STRCopySafe(params[++i],this->name,MAX_NAME);
				else
					return ERR_IPAR;
			}
			else
			if (STRSameCI("-insert",params[i]))
				this->insert=TRUE;
			else
				this->file=params[i];
		}
		if (this->file)
		{
			if (!this->name[0]) STRCopySafe(this->file,this->name,MAX_LINE);
			PWChange(menu,PW_MENU_ADD,this->name,NULL);
		}
	}
	IOClose(chid);
	return PWActivate(obj);
}

Error select(PWObject object, char *item)
{
	Error err;
	Line *g=(Line *)item;
	char *strs[3];
	char *wait;

	PWQuery(object,PW_GLOBAL_AUXILIARY,&wait);
	if (!*wait) PWChange(object,PW_SYSTEM_BREAKDOWN,NULL);

	if (g->insert)
	{
		strs[0]="Please insert disk to load";
		strs[1]=g->name;
		catch( THINGExec("request",8,MYSELF,TRUE,2,strs,0,NULL,NULL) );
	}

	strs[0]="-path";
	strs[1]=g->path;
	strs[2]=g->file;
	err=THINGExec("ProWesS loader",8,g->terminate?SYSTEM:MYSELF,
									 g->terminate?FALSE:TRUE,3,strs,0,NULL,NULL);
	if (g->terminate) JOBZap(MYSELF,ERR_OK);
	if (err)
	{
		char message[ERR_MAXLEN];

		ERRMessageGet(err,message);
		strs[0]="Unable to process";
		strs[1]=g->name;
		strs[2]=message;
		catch( THINGExec("request",8,MYSELF,FALSE,3,strs,0,NULL,NULL) );

	}
	return	ERR_OK;
}
