/*
	utility program for the ProWesS loader

	SETENV : set an environment variable (called "Global Variables").
	This program accepts two parameters, the name of the variable (case
	dependent), and the value.
	If not both a name and a value are passed, ERR_IPAR is returned.
	Otherwise, the error reported when trying to set the var is given.

	For recursive calls, the value is limited to 256 chars.

	written by Joachim Van der Auwera for PROGS
	(c) 1996 by PROGS, Professional & Graphical Software
	started 09 January 1996
	25.01.1996 setenv can now be used in ProWesS loader to automatically set
			   a "Global Variable". If no value was given, then the -path
			   parameter is used as value.
	13.09.1997 -recurse parameter added. When passed and while the value
			   is a gloal variable (starting with a $), then the value is
			   replaced by the value of that global variable.
*/

#include <str.h>
#include <mem.h>
#include <io.h>
#include <thing.h>

#define catch(x) do { if (err=(x)) return err; } while (0)

#define GLOBAL_NAME "Global Variables"
#define GLOBAL_GET	0x47455420	/* "GET " */
#define GLOBAL_SET	0x53455420	/* "SET " */
#define MAX_LEN 256

Error init(char *param)
{
	Error err;
	char **params;
	char *name=NULL, *value=NULL;
	char _value[256], recurse=FALSE;
	int i;

	/* pars the command line parameters */
	catch( STRBreak(param,&params) );
	for (i=0 ; params[i] ; i++)
	{
		if (STRSameCI("-path",params[i]))
		{
			if (params[i+1])
			{
				if (!value) value=params[i+1];
				i++;
			}
			else
				return ERR_IPAR;
		}
		else
		if (STRSameCI("-recurse",params[i]))
			recurse=TRUE;
		else
			if (!name)
				name=params[i];
			else
				value=params[i];
	}
	if (!name || !value) return ERR_IPAR;

	if (recurse)
	{
		while (value[0]=='$')
		{
			catch(THINGCall(GLOBAL_NAME,GLOBAL_GET,3,value+1,_value,MAX_LEN));
			value=_value;
		}
	}

	return THINGCall(GLOBAL_NAME,GLOBAL_SET,2,name,value);
}
